;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x5ddea4b08e7aee052b52f1841346d3d8 . "./code.cg2js/cg-common-control.fasl")
(#x696163c7faa78393fe30970cf09f3f46 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x3a292b18d8100eefa5789bf706f5ac24 . "./code.cg2js/cg-yes-no-string.fasl")
(#x7d70e30835f43cf02e30ac096b44de52 . "./code.cg2js/cg-clipboard.fasl")
(#x643dcca173857f70dfd0f4825332bc3e . "./code.cg2js/cg-dde-utils.fasl")
(#xdfa575fb86575fc864b2536e64e7706e . "./code.cg2js/cg-editable-text.fasl")
(#xb4de3a18470f0361473fcb49d1590f53 . "./code.cg2js/cg-player.fasl")
(#xca2a2afd381430cf1752df5e642c0242 . "./code.cg2js/cg-text-or-combo.fasl")
(#x3bd0dc4a5fc9c5971131e0c581da439d . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x34646502667dc70c93280a2e1b1e556c . "./code.cg2js/cg-graphics-context.fasl")
(#x5058a8a94ee64d2998cee2bdcb21f28f . "./code.cg2js/cg-choose-printer.fasl")
(#x0d7ab1be9607641b368d10274296a8d8 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x77d7b46cac52b5e54998c508f9f92bcd . "./code.cg2js/cg-edit-in-place.fasl")
(#xca84da586e6225595c68e7aed9ae2a24 . "./code.cg2js/cg-split-bar.fasl")
(#x9a6ebb6010b5e1de9dbc79aa8dd0a6c5 . "./code.cg2js/cg-timer.fasl")
(#x4529e4192ec137225d8a714265e989f6 . "./code.cg2js/cg-progress-indicator.fasl")
(#x688feea0c1733dddf58ae7a302aab9a5 . "./code.cg2js/cg-object-editor.fasl")
(#xf84d70627378055d703cde3d7961b985 . "./code.cg2js/cg-fill-texture.fasl")
(#x2bf447d3b39a86bfa1808172bd132bf0 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x8c4c9361d8075f87c4ee39e9e0e0f9b2 . "./code.cg2js/cg-os-window.fasl")
(#x16d7af3c342a948798c376462f95a7dc . "./code.cg2js/cg-color-dialog.fasl")
(#xdf24047bc83be46ce0d580d642c18dfd . "./code.cg2js/cg-combo-box.fasl")
(#x348de9bc89d89d31a722bf523368edfa . "./code.cg2js/cg-directory-dialog.fasl")
(#x81389f41c06edfea9e6ee611a125ddf9 . "./code.cg2js/cg-tooltip.fasl")
(#x4b60672c4716eb52cd9c6958b4626ab8 . "./code.cg2js/cg-lettered-menu.fasl")
(#xcc30eaf9d0449df1ba83df6244f90999 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x14a8c86866db32324d8c1b0e493ec85f . "./code.cg2js/cg-pixmap.fasl")
(#xe53128dbfa44805c1553ea9c535c7cee . "./code.cg2js/cg.fasl")
(#xcb2c06389b2f2ba2c90eaed21e68139f . "./code.cg2js/cg-class-slot-grid.fasl")
(#x4318ddb895895ddf41a58612808633d8 . "./code.cg2js/cg-template-string.fasl")
(#xcfb23895699130abf1dd9cd1031d3631 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xe7d81169643a94f59a50fbfe9b874e0b . "./code.cg2js/cg-check-box.fasl")
(#x3e318bae0923d968fcdc701687ecc97e . "./code.cg2js/cg-toggling-widget.fasl")
(#x1393bd866fd8897865f68e5db6cdd056 . "./code.cg2js/cg-radio-button.fasl")
(#x146dd56df20777c05ba7e1be2dd5e88e . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x6552ce8330e4cdba83af01f1c4db8cec . "./code.cg2js/cg-scaling-stream.fasl")
(#x6db338b2f62150a0392f013c219ea0d8 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xe6e39bb9cb80c0be7309eea0354c24d0 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x9fc7654684b90ea670723d8bbc43db28 . "./code.cg2js/cg-grid-widget.fasl")
(#x49aedc3ae03f8e885730d96b0d202a95 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x23649bb56e40ce97b73d23a6817490f8 . "./code.cg2js/cg-item-list.fasl")
(#xbfec6e1fea80cf7d0f5d984bc80a2520 . "./code.cg2js/cg-paren-matching.fasl")
(#x726e55c47125464d5c0fe25899946c1d . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x19d3fdc9eb8eab7d27237bda14eb8bf7 . "./code.cg2js/cg-html-dialog.fasl")
(#x41f54e97c2799ef1aa97f3ae8fab8818 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x63b90574f0e3833dcc26813c39ec874c . "./code.cg2js/cg-project-window.fasl")
(#x9b0599350bb1cfa54f24c7a2de9ddcc0 . "./code.cg2js/cg-toolbar.fasl")
(#xf25967cf2c2106316b3127959d4ee62b . "./code.cg2js/cg-caret.fasl")
(#xa593e9a642b858f74c842a764a89dce9 . "./code.cg2js/cg-scroll-bar.fasl")
(#xb1c76c558819116c3570c2c744d916fa . "./code.cg2js/cg-web-browser.fasl")
(#x85e21174baa97ac9496202e5b9b06223 . "./code.cg2js/cg-dropping-outline.fasl")
(#x62ab8e064d2165750f0885bd9f04ddd0 . "./code.cg2js/cg-font-dialog.fasl")
(#x26ae606804c7aa64c21dee4dc5a243a7 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x17fda58d8e6c8ed2fe2cb80aa93c8ecd . "./code.cg2js/cg-yes-no-list.fasl")
(#x4fef18bf07937d0d74da7a0530d26528 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xa84ab3a92df47e2901d8b33652e24300 . "./code.cg2js/cg-lamp.fasl")
(#x54e8505986f886f5fbde9da5a7b43962 . "./code.cg2js/cg-drawable.fasl")
(#xfaa816386b930376bddcc4a43c50df42 . "./code.cg2js/cg-choice-list.fasl")
(#xa170d5ca6fb5169b643359fbc87dd4a5 . "./code.cg2js/cg-curve.fasl")
(#xdd7a3501001a6b3b573019012f677ad7 . "./code.cg2js/cg-base.fasl")
(#x20231f9a6d484565cb49d4fee50a9637 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x99c116583430a7b523650d121a4d79d1 . "./code.cg2js/cg-printing.fasl")
(#xc9f841c2302dac5b67a198a99f450958 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x10c48ff17fcc1a4cba66b736b3c723cf . "./code.cg2js/cg-hotspot.fasl")
(#x8b7e16f7bc023434ef91c1b422d7d89d . "./code.cg2js/cg-object-editor-layout.fasl")
(#x9991e9d4efaaf6260a7b790bcd8a56f6 . "./code.cg2js/cg-utility-dialog.fasl")
(#xadff421c27b212dcbdb47549e9ae006b . "./code.cg2js/cg-shortcut-menu.fasl")
(#x33676eacd25507995e07107c8ff50dc2 . "./code.cg2js/cg-comtab.fasl")
(#x68712ef865f38706c3130c56db6e47de . "./code.cg2js/cg-os-widget.fasl")
(#x8ccd7d492dfc6ab4604a52ebbd91f230 . "./code.cg2js/cg-icon.fasl")
(#x98033117a484be1cdc0a481f2fb72c50 . "./code.cg2js/cg-dev.fasl")
(#xb5ff4313ba462b5b9e92db9ba05de5ca . "./code.cg2js/cg-calendar.fasl")
(#x3f5ce3949599844b2e886f650d0864c9 . "./code.cg2js/cg-group-box.fasl")
(#x91280a32c59fedc9970d76824c7fbcda . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x6d3cc2f7453e88b54f0e3c9a68c73ad8 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x71eded4e5fd25611ad6d9c4c86511853 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xf8aad2f693b3c735bdc62c31f484f3b7 . "./code.cg2js/cg-nodes.fasl")
(#xaa1838c8e52e5c004333e3458f1bafff . "./code.cg2js/cg-intersect.fasl")
(#xd7a6cef4a6bcea345f0cfe97ea93f9bb . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x8fc47eba42effe8ba7aeabfb5d69516f . "./code.cg2js/cg-class-support.fasl")
(#x5cfe75c320f9d9d70891eb4645ac95f0 . "./code.cg2js/cg-dialog-item.fasl")
(#x9852e1e756328f0ccd52ce7413f89c99 . "./code.cg2js/cg-plot-widget.fasl")
(#xacacce73312fb767345c4f43f3ffd2ae . "./code.cg2js/cg-pixmap-printing.fasl")
(#x5fc64aa16950716189aa49cbcac7c643 . "./code.cg2js/cg-up-down-control.fasl")
(#x4aa0d563b68620c1991838c25ca85a4c . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xdf95468234388ed3a8b9835ed4b99183 . "./code.cg2js/cg-status-bar.fasl")
(#xc6cb147d02a4f99e7a82eae836b9caaf . "./code.cg2js/cg-class-grid.fasl")
(#xa15b9316ee7b41da5286a0e8d7d79722 . "./code.cg2js/cg-wrap-string.fasl")
(#x9a6f97c5bbd3cc1b93a502da8965b123 . "./code.cg2js/cg-layout.fasl")
(#x97863292d95e7eaf24f1f83551ea9750 . "./code.cg2js/cg-lisp-widget.fasl")
(#x402557f809b82b9a32f5163932a3bf74 . "./code.cg2js/cg-button.fasl")
(#x8df53536f0cc01e213128bb4dc0090b5 . "./code.cg2js/cg-string-dialog.fasl")
(#x6479e36dee428ae560f9b98aa5e5a949 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x507415ecde96d8989449b9b86bca5425 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x7bd4c006d418b58d439297e1d87d7c20 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xec97fc6b79eb50237859dba0a76cf52b . "./code.cg2js/cg-static-text.fasl")
(#xbeab469a5c1191ab83dfd3e210ed2fb5 . "./code.cg2js/cg-outline.fasl")
(#xfb100913650cc8d1465f3a1c702ab5d5 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x6e4536db3076d6fb2aba9fa73a15c5db . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x379ef4ab3256632f525578880d3c0a80 . "./code.cg2js/cg-picture-widget.fasl")
(#xb7d31d3f3ef1a137ff973f401bc11c2a . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x3683fdfae0bd2f4ab278b58c4040c4ed . "./code.cg2js/cg-chart-widget.fasl")
(#x64bc4b4602cd856eee967019cce2c93a . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x4151dbf453d9312ce2818c4affd0c4f9 . "./code.cg2js/cg-lisp-text.fasl")
(#xdfb1847653964604482cfe5825d45d8a . "./code.cg2js/cg-palette.fasl")
(#x52253b654a60b85ae4c64bde67c4bf81 . "./code.cg2js/cg-property.fasl")
(#x6b53cc13861f3db8588d736102863036 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x085b4f9163e37eaec6938f9223e064a4 . "./code.cg2js/cg-file-dialog.fasl")
(#x0d6f0c1e2d3fce9a984f512487d322fc . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x2e353269ac86eee4561d7cf2ae1e46b9 . "./code.cg2js/cg-get-pixmap.fasl")
(#xcaf8686d0ed756d23f6976c4a324ae6e . "./code.cg2js/cg-tray.fasl")
(#x81e0c20573c7204c8f5bd75be4549b99 . "./code.cg2js/cg-acache.fasl")
(#x5ef1eac1ff591f1029154368a0c4634b . "./code.cg2js/cg-message-dialog.fasl")
(#x2095c9b9a45bf1fec2b9ba5dacb6af3e . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xb355c4872a87f55b8ae1cca7e598d7b9 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xe2050d46878c4a54803b570ca9249212 . "./code.cg2js/cg-tab-control.fasl")
(#x94cd250b1aa36d64e5a62c106d17efd8 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x5e5f6712df67066d4c064894e54e6c3e . "./code.cg2js/cg-menu-tooltip.fasl")
(#x3ce5f3e7c8aed9518feec6998b9aa409 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x197f2edca9fe3bc0c9e75304dc19135d . "./code.cg2js/cg-menu.fasl")
(#xa62a1d606ab6f851578298a8a841d1a9 . "./code.cg2js/cg-get-position.fasl")
(#xf0b8ac0ff57956de9d9e954ad191ce0c . "./code.cg2js/cg-trackbar.fasl")
(#xfc27f724e964542d0e446fe3a72a903a . "./code.cg2js/cg-selected-object.fasl")
(#x6ed87c832c630c33bc55e88cdf3e6151 . "./code.cg2js/cg-list-view.fasl")
(#x63461f6a899eacdded735615e5ed7043 . "./code.cg2js/cg-html-widget.fasl")
(#x104841219619baa39219487f9f0561a5 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xa884f2643425d727e50527bcba927a18 . "./code.cg2js/cg-text-widget.fasl")
(#xdc94cb5d75bb3f8a929140e1a2e0bbe0 . "./code.idejs/ide-base.fasl")
(#xf06faec61359cf6c21beb45bf27a9b0c . "./code.idejs/ide-process-browser.fasl")
(#x3ca8246a225b11a53bfeaa41055ac181 . "./code.idejs/ide-patch-dialog.fasl")
(#xe0ee6ce705015d29b37515ee307ba387 . "./code.idejs/ide-keyshort.fasl")
(#xf7416231813e81b65cbc6216fe204300 . "./code.idejs/ide-window-list.fasl")
(#xcc417e50644b6468b92f9763636bf99d . "./code.idejs/ide.fasl")
(#xa1663aed9a8d71db8b290bfdb9b24dcf . "./code.idejs/ide-editor.fasl")
(#x8ef537eb56730c3460bc6ad7edfd8632 . "./code.idejs/ide-clipboard.fasl")
(#x1b47aa115718b093aae6e9b4b48007dc . "./code.idejs/ide-trace.fasl")
(#xe32fc94175983a85ab76fa48305c2751 . "./code.idejs/ide-class-browser.fasl")
(#xf9d01f2f178f49fa9646ddee4f6b87dd . "./code.idejs/ide-grapher.fasl")
(#x3efde685ec829d937089229452f7aee9 . "./code.idejs/ide-apropos.fasl")
(#xabb7745400824ac217ca93f0358d8cde . "./code.idejs/ide-navigator.fasl")
(#x442e4d29ee8e58939a7132cb841b174c . "./code.idejs/ide-find-definitions.fasl")
(#x210f13f67d039dc0aee9473dbb4854eb . "./code.idejs/ide-project.fasl")
(#xf2441b72412365c3f322c6cdb81c1787 . "./code.idejs/ide-class-grapher.fasl")
(#x666b95a6f90881b7a7ca0338859a4042 . "./code.idejs/ide-acache.fasl")
(#xece612e375946f715386e3e36c55ae53 . "./code.idejs/ide-find-in-files.fasl")
(#xdb4ccf0052db609f1584d77ac466cf44 . "./code.idejs/ide-inspector.fasl")
(#x38be051fe6266303b4f00aa3f707f3ed . "./code.idejs/ide-intro.fasl")
(#xb379737c6f2ef19bfec6676739119671 . "./code.idejs/ide-package-list.fasl")
(#xdadc00457302b4c8d43a42e1c15e22ba . "./code.idejs/ide-dev.fasl")
(#xac43073f42d86294c471005a8750cb9e . "./code.idejs/ide-form.fasl")
(#x7e8a653f63c59e56c07642baa40b4a7f . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
